clear all
close all
clc
set(0,'DefaultFigureWindowStyle','docked')

%% load argentine data
% I assume everything is end of year or the end of the quarter

% time period is 1993-2001

data_annual = xlsread('arg_data.xlsx','annual');

data.annual.ts = data_annual(23:32,1); % 1992:2001, year end
data.annual.debt_to_gdp = data_annual(23:32,3); % b/y

% convert into quarters

data_qtr = xlsread('arg_data.xlsx','quarterly');
data.quarter.ts = (1993:0.25:2002); % 1992:Q4 to 2001Q4
data.quarter.gdp = exp(data_qtr(52:end,1)); % exp(z)
data.quarter.spreads = data_qtr(52:end,2);

% convert to months
data.month.ts = 1993:(1/12):2002;
% data.month.gdp = interp1(data.quarter.ts,data.quarter.gdp,data.month.ts,'linear');
data.month.gdp = interp1(data.quarter.ts,data.quarter.gdp,data.month.ts,'linear');
data.month.spreads = interp1(data.quarter.ts,data.quarter.spreads,data.month.ts,'linear');

%% load model

filename = 'baseline_11042020.mat';
load(filename,'p','model')

%% compute spreads

spreads_total = (1 + (p.debt_m + (1-p.debt_m)*p.debt_z)./model.qNDH - p.debt_m).^12 - (1 + p.rH).^12;

% 2 term decomposition
spreads_DEF = (1 + (p.debt_m + (1-p.debt_m)*p.debt_z)./model.qND_noliq - p.debt_m).^12 - (1 + p.rH).^12;
spreads_LIQ = spreads_total - spreads_DEF;

%% fit tfp shocks (nearest neighbor)

gdp_fitted = zeros(size(data.month.ts));
idx_gdp_fitted = zeros(size(data.month.ts));

for i = 1:numel(gdp_fitted)
    [~,idx_gdp_fitted(i)] = min(abs(p.grid_y_nd-data.month.gdp(i)));
    gdp_fitted(i) = p.grid_y_nd(idx_gdp_fitted(i));
end

%% get path

% mval = p.ggq_mmean;
mval = p.ggq_mub;
b_model_fitted = zeros(size(data.month.ts));
qH_model_fitted = zeros(size(data.month.ts)); qH_model_fitted(end) = NaN;
qL_model_fitted = zeros(size(data.month.ts)); qL_model_fitted(end) = NaN;
idxb_model_fitted = zeros(size(data.month.ts));

spreads_total_fitted = NaN*zeros(size(data.month.ts));
spreads_DEF_fitted = NaN*zeros(size(data.month.ts));
spreads_LIQ_fitted = NaN*zeros(size(data.month.ts));

default_indicator = zeros(size(data.month.ts));
default_indicator(1) = false;

% initial value
MULT = 3; % monthly conversion
% [~,idxb_model_fitted(1)] = min(abs(p.grid_b-MULT*data.annual.debt_to_gdp(1)*data.quarter.gdp(1)));
% b_model_fitted(1) = p.grid_b(idxb_model_fitted(1));

spread_init = ((p.debt_m +(1-p.debt_m)*p.debt_z ...
    + (1-p.debt_m)*model.qNDH(idx_gdp_fitted(1),:))./model.qNDH(idx_gdp_fitted(1),:)).^(4*MULT) - (1+p.rH).^(4*MULT);

[~,idxb_model_fitted(1)] = min(abs(spread_init-data.quarter.spreads(1)/100));
b_model_fitted(1) = p.grid_b(idxb_model_fitted(1));

for i = 2:numel(idxb_model_fitted)
    % find debt policy b' at t=i-1
    temp_policy = model.policy{idx_gdp_fitted(i-1),idxb_model_fitted(i-1)};
    idx_region = find((mval>=temp_policy.eps_mesh(1:end-1)).*(mval<=temp_policy.eps_mesh(2:end)),...
        1,'first');
    idx_bnext = temp_policy.idx_bnext(idx_region);
    idxb_model_fitted(i) = idx_bnext;
    b_model_fitted(i) = p.grid_b(idx_bnext);
    default_indicator(i) = model.defthresh(idx_gdp_fitted(i),idx_bnext) >= mval;
    qH_model_fitted(i-1) = model.qNDH(idx_gdp_fitted(i-1),idx_bnext);
    qL_model_fitted(i-1) = model.qNDL(idx_gdp_fitted(i-1),idx_bnext);
    
    % use version 2
    spreads_total_fitted(i-1) = spreads_total(idx_gdp_fitted(i-1),idx_bnext);
    spreads_DEF_fitted(i-1) = spreads_DEF(idx_gdp_fitted(i-1),idx_bnext);
    spreads_LIQ_fitted(i-1) = spreads_LIQ(idx_gdp_fitted(i-1),idx_bnext);

end

%% plot

m = 2; n = 3;

figure

subplot(m,n,1)
hold on
plot(data.quarter.ts,data.quarter.gdp,'-')
plot(data.month.ts,gdp_fitted,'-d')
axis square
grid on
xlabel('t')
ylabel('y=e^z')
legend({'data','fitted (nearest point)'})

subplot(m,n,2)
hold on
plot(data.annual.ts + 1,data.annual.debt_to_gdp,'-o')
plot(data.month.ts,(1/3)*b_model_fitted./gdp_fitted,'-d')
grid on
axis square
xlabel('t')
ylabel('b/y (year-end)')
legend({'data (annual)','fitted (quarterly, conditional on ND)'})

subplot(m,n,3)
hold on
plot(data.annual.ts + 1,data.annual.debt_to_gdp.*data.quarter.gdp(1:4:numel(data.quarter.gdp)),'-o')
plot(data.month.ts,b_model_fitted/3,'-d')
grid on
axis square
xlabel('t')
ylabel('b')
legend({'data (annual)','fitted (quarterly, conditional on ND)'})

subplot(m,n,4)
plot(data.month.ts + 1,default_indicator,'-d')
grid on
axis square
xlabel('t')
ylabel('default?')
legend({'default=1 (model implied default)'})
set(gca,'ylim',[0 1])

subplot(m,n,5)
hold on
plot(data.quarter.ts + 1,data.quarter.spreads,'-o')
spreads_model = ((p.debt_m +(1-p.debt_m)*p.debt_z ...
    + (1-p.debt_m)*qH_model_fitted)./qH_model_fitted).^(4*MULT) - (1+p.rH).^(4*MULT);
plot(data.month.ts + 1,100*spreads_model,'-d')
grid on
axis square
xlabel('t')
ylabel('spreads (%)')
legend({'data','model (based on ND)'})

subplot(m,n,6)
qsale = p.bargain_power_mkt_maker_ND*qL_model_fitted + (1-p.bargain_power_mkt_maker_ND)*qH_model_fitted;
bidask_model = 2*(qH_model_fitted-qsale)./(qH_model_fitted+qsale);
plot(data.month.ts + 1,100*bidask_model,'-d')
grid on
axis square
xlabel('t')
ylabel('bid-ask spread (%)')

% comment:
% it looks like we don't see a feedback effect in the time series
% this is because of endogenous policy choice
% rather, the choice 

%% plot: spreads

figure

subplot(1,2,1)
hold on
plot(data.quarter.ts,data.quarter.spreads,'-o')
spreads_model = ((p.debt_m +(1-p.debt_m)*p.debt_z ...
    + (1-p.debt_m)*qH_model_fitted)./qH_model_fitted).^(4*MULT) - (1+p.rH).^(4*MULT);
spreads_model(default_indicator==1) = NaN;
plot(data.month.ts,100*spreads_model,'-d')
grid on
axis square
% xlabel('t')
ylabel('spreads (%)','FontSize',12)
legend({'data','model'},'Location','NorthWest','FontSize',12)

subplot(1,2,2)
qsale = p.bargain_power_mkt_maker_ND*qL_model_fitted + (1-p.bargain_power_mkt_maker_ND)*qH_model_fitted;
bidask_model = 2*(qH_model_fitted-qsale)./(qH_model_fitted+qsale);
bidask_model(default_indicator==1) = NaN;
plot(data.month.ts + 1,100*bidask_model,'-d')
grid on
axis square
xlabel('t')
ylabel('bid-ask spread (%)')

% print -dpdf spreads.pdf

%% decomposition

lgdFontSize = 12;

figure

hold on

plot(data.quarter.ts,data.quarter.spreads,'-o')

spreads_total_fitted(default_indicator==1) = NaN;
plot(data.month.ts,100*spreads_total_fitted,'-d')

spreads_DEF_fitted(default_indicator==1) = NaN;
plot(data.month.ts,100*spreads_DEF_fitted,'-d')

spreads_LIQ_fitted(default_indicator==1) = NaN;
plot(data.month.ts,100*spreads_LIQ_fitted,'-d')

grid on
axis square
% xlabel('t')
ylabel('spreads (%)','FontSize',12)
legend({'data','model, total','model, DEF','model, LIQ'},'Location','NorthWest','FontSize',12)

%% coarse decomposition, with output

FontSize = 18;
% lgdFontSize = 14;
lgdFontSize = 18;
LineWidth = 2;

figure

subplot(1,3,1)
hold on
plot(data.quarter.ts,data.quarter.gdp,'-d','LineWidth',LineWidth)
plot(data.month.ts,gdp_fitted,'x','LineWidth',LineWidth)
axis square
% grid on
box on
title('A. Output','Interpreter','Latex','FontSize',FontSize)
legend({'Data','Model (fitted)'},'Interpreter','Latex','FontSize',lgdFontSize,'Location','SouthWest')
legend('boxoff')

xtickpts = 1993:3:2002;

set(gca,'xlim',[1993 2002])
set(gca,'xtick',xtickpts)
set(gca,'xticklabel',xtickpts)

idx_plot = find(spreads_total_fitted<=1);

subplot(1,3,2)
hold on
plot(data.quarter.ts,data.quarter.spreads,'-b','LineWidth',LineWidth)
box on
spreads_total_fitted(default_indicator==1) = NaN;
plot(data.month.ts(idx_plot),100*spreads_total_fitted(idx_plot),'-.r','LineWidth',LineWidth)
axis square
set(gca,'ylim',[0 30])

set(gca,'xlim',[1993 2002])
set(gca,'xtick',xtickpts)
set(gca,'xticklabel',xtickpts)

legend({'Data','Model'},'Interpreter','Latex','FontSize',lgdFontSize,'Location','NorthWest')
legend('boxoff')
% grid on

title('B. Spreads, data vs. model','Interpreter','Latex','FontSize',FontSize)
ylabel('Percent (\%)','Interpreter','Latex','FontSize',FontSize)

subplot(1,3,3)
hold on
box on
spreads_total_fitted(default_indicator==1) = NaN;
plot(data.month.ts(idx_plot),100*spreads_total_fitted(idx_plot),'-.r','LineWidth',LineWidth)

spreads_DEF_fitted(default_indicator==1) = NaN;
plot(data.month.ts(idx_plot),100*spreads_DEF_fitted(idx_plot),'--m','LineWidth',LineWidth)

spreads_LIQ_fitted(default_indicator==1) = NaN;
plot(data.month.ts(idx_plot),100*spreads_LIQ_fitted(idx_plot),':k','LineWidth',LineWidth)
axis square

set(gca,'xlim',[1993 2002])
set(gca,'xtick',xtickpts)
set(gca,'xticklabel',xtickpts)
set(gca,'ylim',[0 30])

legend({'Total','Default','Liquidity'},'Interpreter','Latex','FontSize',lgdFontSize,'Location','NorthWest')
legend('boxoff')
% grid on

title('C. Spreads, decomposition','Interpreter','Latex','FontSize',FontSize)
ylabel('Percent (\%)','Interpreter','Latex','FontSize',FontSize)

set(gca,'ylim',[0 30])

h=gcf;
set(h,'PaperOrientation','landscape');
set(h,'PaperUnits','normalized');
set(h,'PaperPosition', [0 0 1 1]);
% print(gcf, '-depsc2', 'spreads_decomposition_argentina_1x3.eps');

figure
plot(data.month.ts,spreads_LIQ_fitted./spreads_total_fitted,'-b','LineWidth',LineWidth)
set(gca,'xlim',[1993 2002])
set(gca,'xtick',xtickpts)
set(gca,'xticklabel',xtickpts)
title(sprintf('Liquidity share of total spread. Average: %g.',nanmean(spreads_LIQ_fitted./spreads_total_fitted)))

%% coarse decomposition, with output, 1 by 2

% LineWidth = 2;
% 
% figure
% 
% subplot(1,2,1)
% 
% yyaxis right
% hold on
% plot(data.quarter.ts,data.quarter.gdp,'-r','LineWidth',1.5)
% plot(data.month.ts,gdp_fitted,'xr','LineWidth',1.5)
% axis square
% grid on
% % title('A. Output','Interpreter','Latex','FontSize',12)
% % legend({'Data','Model (fitted)'},'Interpreter','Latex','FontSize',lgdFontSize,'Location','SouthWest')
% % legend('boxoff')
% ylabel('Output','Interpreter','Latex','FontSize',12)
% 
% yyaxis left
% hold on
% plot(data.quarter.ts,data.quarter.spreads,'-b','LineWidth',LineWidth)
% 
% spreads_total_fitted(default_indicator==1) = NaN;
% plot(data.month.ts,100*spreads_total_fitted,'-.b','LineWidth',LineWidth)
% axis square
% 
% set(gca,'xlim',[1993 2002])
% set(gca,'xtick',xtickpts)
% set(gca,'xticklabel',xtickpts)
% 
% legend({'Spreads (data)','Spreads (model)','Output (data)','Output (model)'},'Interpreter','Latex','FontSize',lgdFontSize,'Location','NorthWest')
% legend('boxoff')
% grid on
% 
% title('A. Spreads, data vs. model','Interpreter','Latex','FontSize',12)
% ylabel('Spreads, (\%)','Interpreter','Latex','FontSize',12)
% 
% subplot(1,2,2)
% hold on
% 
% yyaxis right
% hold on
% plot(data.quarter.ts,data.quarter.gdp,'-r','LineWidth',1.5)
% plot(data.month.ts,gdp_fitted,'xr','LineWidth',1.5)
% axis square
% grid on
% % title('A. Output','Interpreter','Latex','FontSize',12)
% % legend({'Data','Model (fitted)'},'Interpreter','Latex','FontSize',lgdFontSize,'Location','SouthWest')
% % legend('boxoff')
% ylabel('Output','Interpreter','Latex','FontSize',12)
% 
% yyaxis left
% 
% spreads_total_fitted(default_indicator==1) = NaN;
% plot(data.month.ts,100*spreads_total_fitted,'-.b','LineWidth',LineWidth)
% 
% spreads_DEF_fitted(default_indicator==1) = NaN;
% plot(data.month.ts,100*spreads_DEF_fitted,'--b','LineWidth',LineWidth)
% 
% spreads_LIQ_fitted(default_indicator==1) = NaN;
% plot(data.month.ts,100*spreads_LIQ_fitted,':b','LineWidth',LineWidth)
% axis square
% 
% set(gca,'xlim',[1993 2002])
% set(gca,'xtick',xtickpts)
% set(gca,'xticklabel',xtickpts)
% set(gca,'ylim',[0 30])
% 
% legend({'Total','Default','Liquidity'},'Interpreter','Latex','FontSize',lgdFontSize,'Location','NorthWest')
% legend('boxoff')
% grid on
% 
% title('B. Spreads, model decomposition','Interpreter','Latex','FontSize',12)
% ylabel('Percent (\%)','Interpreter','Latex','FontSize',12)
% 
% set(gca,'ylim',[0 30])
% 
% h=gcf;
% set(h,'PaperOrientation','landscape');
% set(h,'PaperUnits','normalized');
% set(h,'PaperPosition', [0 0 1 1]);
% print(gcf, '-depsc2', 'spreads_decomposition_argentina_1x2_v2.eps');