clear all
% close all
clc
set(0,'DefaultFigureWindowStyle','docked')

%% specify file

load('baseline_no_liq_11042020.mat','p_no_liq','model_no_liq')
pCE = p_no_liq;
modelCE = model_no_liq;

clear p_no_liq model_no_liq

load('baseline_11042020.mat','p','model')

load('decomposition.mat','q_b0_d1_noliq','q_b1_d0_noliq')

bPrint = false;
% bPrint = true;

%% decomposition is for a fixed value of y

% yval = 0.9;
yval = 1;
% yval = 1.1;

[~,idxy] = min(abs(yval - p.grid_y_nd));

%% get all relevant terms

spreads_total = (1 + (p.debt_m + (1-p.debt_m)*p.debt_z)./model.qNDH - p.debt_m).^12 - (1 + p.rH).^12;

% 2 term decomposition
spreads_DEF = (1 + (p.debt_m + (1-p.debt_m)*p.debt_z)./model.qND_noliq - p.debt_m).^12 - (1 + p.rH).^12;
spreads_LIQ = spreads_total - spreads_DEF;

% 4 term decomposition
spreads_pureDEF = (1 + (p.debt_m + (1-p.debt_m)*p.debt_z)./modelCE.qNDH - p.debt_m).^12 - (1 + p.rH).^12;
spreads_LIQ_DEF = spreads_DEF - spreads_pureDEF;

% compute pure liquidity price
btemp = [p.debt_m + (1-p.debt_m)*p.debt_z;p.debt_m + (1-p.debt_m)*p.debt_z-p.holding_cost];
Atemp = [1+p.rH-(1-p.debt_m)*(1-p.prob_liqshock),-(1-p.debt_m)*p.prob_liqshock;...
    -(1-p.debt_m)*p.prob_meet_mkt_maker_ND*(1-p.bargain_power_mkt_maker_ND),1+p.rL-(1-p.debt_m)*(1-p.prob_meet_mkt_maker_ND*(1-p.bargain_power_mkt_maker_ND))];
qs_pureLIQ = Atemp\btemp;

spreads_pureLIQ = ((1 + (p.debt_m + (1-p.debt_m)*p.debt_z)/qs_pureLIQ(1) - p.debt_m).^12 - (1 + p.rH).^12)*ones(p.gridsize_tfp,p.gridsize_b);

spreads_DEF_LIQ = spreads_LIQ - spreads_pureLIQ;

spreads_b0_d1_noliq = (1 + (p.debt_m + (1-p.debt_m)*p.debt_z)./q_b0_d1_noliq - p.debt_m).^12 - (1 + p.rH).^12;
spreads_b1_d0_noliq = (1 + (p.debt_m + (1-p.debt_m)*p.debt_z)./q_b1_d0_noliq - p.debt_m).^12 - (1 + p.rH).^12;

spreads_LIQ_DEFPOL_DEF = spreads_b0_d1_noliq - spreads_pureDEF;
spreads_LIQ_DEBTPOL_DEF = spreads_DEF - spreads_b0_d1_noliq;

save('decomposition_6term.mat')

%% plot, 1x3

idx_selected = find(model.defprob(idxy,:)<=p.max_def_prob);

xlim = [0 4];

LineWidth = 2;
FontSize = 18;
% LegendFontSize = 14;
LegendFontSize = 18;

% need to take choice into account
spreads_total_temp = NaN*zeros(p.gridsize_b,1);
spreads_DEF_temp = NaN*zeros(p.gridsize_b,1);
spreads_LIQ_temp = NaN*zeros(p.gridsize_b,1);
spreads_pureDEF_temp = NaN*zeros(p.gridsize_b,1);
spreads_LIQ_DEF_temp = NaN*zeros(p.gridsize_b,1);
spreads_LIQ_DEFPOL_DEF_temp = NaN*zeros(p.gridsize_b,1);
spreads_LIQ_DEBTPOL_DEF_temp = NaN*zeros(p.gridsize_b,1);
spreads_pureLIQ_temp = NaN*zeros(p.gridsize_b,1);
spreads_DEF_LIQ_temp = NaN*zeros(p.gridsize_b,1);

m0 = p.ggq_mmean;

for i = 1:p.gridsize_b
    if model.defthresh(idxy,i)<m0
        idx_region = find(model.policy{idxy,i}.eps_mesh(1:end-1)<=m0 & m0<=model.policy{idxy,i}.eps_mesh(2:end),1,'first');
        idxbnext = model.policy{idxy,i}.idx_bnext(idx_region);
        spreads_total_temp(i) = spreads_total(idxy,idxbnext);
        spreads_DEF_temp(i) = spreads_DEF(idxy,idxbnext);
        spreads_LIQ_temp(i) = spreads_LIQ(idxy,idxbnext);
        spreads_pureDEF_temp(i) = spreads_pureDEF(idxy,idxbnext);
        spreads_LIQ_DEF_temp(i) = spreads_LIQ_DEF(idxy,idxbnext);
        spreads_LIQ_DEFPOL_DEF_temp(i) = spreads_LIQ_DEFPOL_DEF(idxy,idxbnext);
        spreads_LIQ_DEBTPOL_DEF_temp(i) = spreads_LIQ_DEBTPOL_DEF(idxy,idxbnext);
        spreads_pureLIQ_temp(i) = spreads_pureLIQ(idxy,idxbnext);
        spreads_DEF_LIQ_temp(i) = spreads_DEF_LIQ(idxy,idxbnext);
    end
end

figure

subplot(1,3,1)
hold on
plot(p.grid_b/3,spreads_total_temp,'-b','LineWidth',LineWidth)
plot(p.grid_b/3,spreads_DEF_temp,'-.r','LineWidth',LineWidth)
plot(p.grid_b/3,spreads_LIQ_temp,'--k','LineWidth',LineWidth)
axis square
title('A. Spreads','Interpreter','Latex','FontSize',FontSize)
legend({'Total','Default','Liquidity'},'Interpreter','Latex','FontSize',LegendFontSize,'Location','NorthWest')
% grid on
legend('boxoff')
set(gca,'xlim',[0 1.3])
set(gca,'xtick',0:.25:1.25)
set(gca,'xticklabel',0:.25:1.25)
% set(gca,'ylim',[0 0.5])
xlabel('Debt level, $b/3$','Interpreter','Latex','FontSize',FontSize)
% set(gca,'ytick',0:0.1:0.3)
% set(gca,'yticklabel',0:0.1:0.3)

subplot(1,3,2)
hold on
plot(p.grid_b/3,spreads_pureDEF_temp,'-b','LineWidth',LineWidth)
plot(p.grid_b/3,spreads_LIQ_DEF_temp,'-.r','LineWidth',LineWidth)
plot(p.grid_b/3,spreads_LIQ_DEFPOL_DEF_temp,'--k','LineWidth',LineWidth)
plot(p.grid_b/3,spreads_LIQ_DEBTPOL_DEF_temp,':m','LineWidth',LineWidth)
axis square
title('B. Default component','Interpreter','Latex','FontSize',FontSize)
legend({'Pure def','Liq$\rightarrow$def',...
    ['Liq$\rightarrow$def,',char(10),'def policy'],...
    ['Liq$\rightarrow$debt,',char(10),'debt policy']},'Interpreter','Latex','FontSize',LegendFontSize,'Location','NorthWest')
% grid on
legend('boxoff')
set(gca,'xlim',[0 1.3])
set(gca,'xtick',0:.25:1.25)
set(gca,'xticklabel',0:.25:1.25)
% set(gca,'ylim',[-0.1 0.5])
xlabel('Debt level, $b/3$','Interpreter','Latex','FontSize',FontSize)
% set(gca,'ytick',-0.3:0.1:0.3)
% set(gca,'yticklabel',-0.3:0.1:0.3)

subplot(1,3,3)
hold on
plot(p.grid_b/3,spreads_pureLIQ_temp,'-b','LineWidth',LineWidth)
plot(p.grid_b/3,spreads_DEF_LIQ_temp,'-.r','LineWidth',LineWidth)
axis square
title('C. Liquidity component','Interpreter','Latex','FontSize',FontSize)
% legend({'Pure liquidity','Liq$\rightarrow$def','Liq$\rightarrow$def policy$\rightarrow$def','Liq$\rightarrow$debt policy$\rightarrow$def'},'Interpreter','Latex','FontSize',LegendFontSize,'Location','NorthWest')
% grid on
legend('boxoff')
set(gca,'xlim',[0 1.3])
set(gca,'xtick',0:.25:1.25)
set(gca,'xticklabel',0:.25:1.25)
xlabel('Debt level, $b/3$','Interpreter','Latex','FontSize',FontSize)
% set(gca,'ylim',[0 0.3])
legend({'Pure liq','Def$\rightarrow$liq'},'Interpreter','Latex','FontSize',LegendFontSize,'Location','NorthWest')
legend('boxoff')

if bPrint
    h=gcf;
    set(h,'PaperOrientation','landscape');
    set(h,'PaperUnits','normalized');
    set(h,'PaperPosition', [0 0 1 1]);
    print(gcf, '-depsc2', 'spreads_decomposition_6term_1x3_v2.eps');
end