set(0,'DefaultFigureWindowStyle','docked')
clear all

load('baseline_11042020.mat','p','model')

load('baseline_no_liq_11042020.mat','p_no_liq','model_no_liq')

%% compare debt and default policies with and without liquidity

% default threshold
ydefthresh = NaN*zeros(p.gridsize_b,1);
ydefthresh_no_liq = NaN*zeros(p.gridsize_b,1);

for i = 1:p.gridsize_b
    idx = find(model.defthresh(:,i)>=p.ggq_mmean,1,'last');
    if ~isempty(idx)
        ydefthresh(i) = p.grid_y_nd(idx);
    end
    idx = find(model_no_liq.defthresh(:,i)>=p.ggq_mmean,1,'last');
    if ~isempty(idx)
        ydefthresh_no_liq(i) = p.grid_y_nd(idx);
    end
end

% debt policy for a fixed yval

% yval = 1;
yval = 1.1;
[~,idx_yval] = min(abs(yval - p.grid_y_nd));
bnext = NaN*zeros(p.gridsize_b,1);
bnext_no_liq = NaN*zeros(p.gridsize_b,1);

for i = 1:p.gridsize_b
    if model.defthresh(idx_yval,i)<p.ggq_mmean
        idx = find(model.policy{idx_yval,i}.eps_mesh(1:end-1)<=p.ggq_mmean & ...
            p.ggq_mmean<=model.policy{idx_yval,i}.eps_mesh(2:end),1,'first');
        bnext(i) = p.grid_b(model.policy{idx_yval,i}.idx_bnext(idx));
    end
    if model_no_liq.defthresh(idx_yval,i)<p.ggq_mmean
        idx = find(model_no_liq.policy{idx_yval,i}.eps_mesh(1:end-1)<=p.ggq_mmean & ...
            p.ggq_mmean<=model_no_liq.policy{idx_yval,i}.eps_mesh(2:end),1,'first');
        bnext_no_liq(i) = p.grid_b(model_no_liq.policy{idx_yval,i}.idx_bnext(idx));
    end
end

% debt policy for fixed bval
bval = 3;
[~,idx_bval] = min(abs(bval - p.grid_b));
bnext2 = NaN*zeros(p.gridsize_tfp,1);
bnext_no_liq2 = NaN*zeros(p.gridsize_tfp,1);

for i = 1:p.gridsize_tfp
    if model.defthresh(i,idx_bval)<p.ggq_mmean
        idx = find(model.policy{i,idx_bval}.eps_mesh(1:end-1)<=p.ggq_mmean & ...
            p.ggq_mmean<=model.policy{i,idx_bval}.eps_mesh(2:end),1,'first');
        bnext2(i) = p.grid_b(model.policy{i,idx_bval}.idx_bnext(idx));
    end
    if model_no_liq.defthresh(i,idx_bval)<p.ggq_mmean
        idx = find(model_no_liq.policy{i,idx_bval}.eps_mesh(1:end-1)<=p.ggq_mmean & ...
            p.ggq_mmean<=model_no_liq.policy{i,idx_bval}.eps_mesh(2:end),1,'first');
        bnext_no_liq2(i) = p.grid_b(model_no_liq.policy{i,idx_bval}.idx_bnext(idx));
    end
end

%% plot

figure

subplot(1,3,1)
hold on
plot(p.grid_b/3,ydefthresh)
plot(p.grid_b/3,ydefthresh_no_liq)
xlabel('b/3')
ylabel('default threshold: y')
grid on
legend({'baseline','no liquidity'},'location','NorthWest')
legend boxoff
set(gca,'xlim',[0.8 1.2])

subplot(1,3,2)
hold on
plot(p.grid_b/3,bnext - (1-p.debt_m)*p.grid_b)
plot(p.grid_b/3,bnext_no_liq - (1-p.debt_m)*p.grid_b)
xlabel('b/3')
ylabel('b'', given y')
grid on
legend({'baseline','no liquidity'},'location','NorthWest')
legend boxoff
% set(gca,'xlim',[0.8 1.2])

subplot(1,3,3)
hold on
plot(p.grid_y_nd,bnext2 - (1-p.debt_m)*p.grid_b(idx_bval))
plot(p.grid_y_nd,bnext_no_liq2 - (1-p.debt_m)*p.grid_b(idx_bval))
xlabel('y')
ylabel('b'', given b')
grid on
legend({'baseline','no liquidity'},'location','NorthWest')
legend boxoff
set(gca,'xlim',[0.95 1.05])

%% nice plots

LineWidth = 2;
FontSize = 18;
% LegendFontSize = 14;
LegendFontSize = 18;

figure

subplot(1,2,2)
hold on
plot(p.grid_b/3,3*ydefthresh,'-b','LineWidth',LineWidth)
plot(p.grid_b/3,3*ydefthresh_no_liq,'--r','LineWidth',LineWidth)
xlabel('Debt level, $b/3$','Interpreter','Latex','FontSize',FontSize)
title('B. Default threshold, $3y_{def}$','Interpreter','Latex','FontSize',FontSize)
legend({'Baseline','No liquidity'},'location','NorthWest','Interpreter','Latex','FontSize',FontSize)
legend boxoff
set(gca,'xlim',[0.8 1.2])

subplot(1,2,1)
hold on
plot(3*p.grid_y_nd,(bnext2 - (1-p.debt_m)*p.grid_b(idx_bval)),'-b','LineWidth',LineWidth)
plot(3*p.grid_y_nd,(bnext_no_liq2 - (1-p.debt_m)*p.grid_b(idx_bval)),'--r','LineWidth',LineWidth)
xlabel('Quarterly output, $3y$','Interpreter','Latex','FontSize',FontSize)
title('A. Debt issuance, $b''-(1-m)b$','Interpreter','Latex','FontSize',FontSize)
% grid on
legend({'Baseline','No liquidity'},'location','NorthWest','Interpreter','Latex','FontSize',FontSize)
legend boxoff
set(gca,'xlim',[2.85 3.15])
% set(gca,'xtick',0.95:0.02:1.05)
% set(gca,'xticklabel',0.95:0.02:1.05)
% set(gca,'ylim',[0.006 0.022])
% set(gca,'ytick',0.006:0.004:0.022)
% set(gca,'yticklabel',0.006:0.004:0.022)