function [qND_H,qND_L,qS_ND,qD_H,qD_L,qS_D,check] = CalcBondPrices_ShortRun_LCP(p,pdef,qND_H_LR,qND_L_LR)

    B = zeros(6,1);
    A = zeros(6,6);
    
    B(1) = (1-pdef)*(p.m + (1-p.m)*p.z)/(1 + p.r);
    B(2) = (1-pdef)*(p.m + (1-p.m)*p.z)/(1 + p.r) - p.hc/(1 + p.r);
    B(3) = p.theta*p.f*qND_H_LR;
    B(4) = -(1-p.theta)*p.hc/(1 + p.r) + p.theta*p.f*qND_L_LR;
    
    A(1,1) = (1 - pdef)*(1 - p.m)*(1 - p.zeta)/(1 + p.r);
    A(1,2) = (1 - pdef)*(1 - p.m)*p.zeta/(1 + p.r);
    A(1,3) = pdef*(1 - p.zeta)/(1 + p.r);
    A(1,4) = pdef*p.zeta/(1 + p.r);
    
    A(2,2) = (1 - pdef)*(1 - p.m)*(1 - p.lambda)/(1 + p.r);
    A(2,4) = pdef*(1 - p.lambda)/(1 + p.r);
    A(2,5) = (1 - pdef)*(1 - p.m)*p.lambda/(1 + p.r);
    A(2,6) = pdef*p.lambda/(1 + p.r);
    
    A(3,3) = (1 - p.theta)*(1 - p.zeta)/(1 + p.r);
    A(3,4) = (1 - p.theta)*p.zeta/(1 + p.r);
    
    A(4,4) = (1 - p.theta)*(1 - p.lambda)/(1 + p.r);
    A(4,6) = (1 - p.theta)*p.lambda/(1 + p.r);
    
    A(5,1) = p.alphaND;
    A(5,2) = 1 - p.alphaND;
    
    A(6,3) = p.alphaD;
    A(6,4) = 1 - p.alphaD;
    
    q = LCP(eye(6,6) - A,-B);
    
    if any(q<0)
        q(:) = NaN;
    end
    
    qND_H = q(1);
    qND_L = q(2);
    qD_H = q(3);
    qD_L = q(4);
    qS_ND = q(5);
    qS_D = q(6);
    
    % check 
    errs = zeros(6,1);
    
    if all(q>0)
    
        errs(1) = ((1-pdef)*(p.m + (1-p.m)*(p.z + p.zeta*qND_L + (1-p.zeta)*qND_H)) ...
            + pdef*(p.zeta*qD_L + (1-p.zeta)*qD_H))/(1 + p.r) - qND_H;

        errs(2) = ((1-pdef)*(p.m - p.hc + (1-p.m)*(p.z + p.lambda*qS_ND + (1-p.lambda)*qND_L)) ...
            + pdef*(-p.hc + p.lambda*qS_D + (1-p.lambda)*qD_L))/(1 + p.r) - qND_L;

        errs(3) = (1-p.theta)*(p.zeta*qD_L + (1-p.zeta)*qD_H)/(1 + p.r) + p.theta*p.f*qND_H_LR - qD_H;
        errs(4) = (1-p.theta)*(-p.hc + p.lambda*qS_D + (1-p.lambda)*qD_L)/(1 + p.r) + p.theta*p.f*qND_L_LR - qD_L;

        errs(5) = p.alphaND*qND_H + (1-p.alphaND)*qND_L - qS_ND;
        errs(6) = p.alphaD*qD_H + (1-p.alphaD)*qD_L - qS_D;
    
    end
    
    check = max(abs(errs));
    
    if check>1e-2; 
        warning('Check CalcBondPrices_ShortRun_LCP'); 
    end

end