% close all
clear all
clc
set(0,'DefaultFigureWindowStyle','docked')

%parameters
p.m       = 0.0167 ;
p.z       = 0.01;
% p.z       = 0;
p.theta   = 0.0128; % probability of reentry
p.r       = 0.0033;
p.hc      = 0.0062;
p.zeta    = 0.1121; % probability of a liquidity shock
p.lambda  = 0.8647; % probability of meeting a market maker

% p.alphaND_LR = 0.125;
% p.alphaD_LR  = 0.0;

% p.alphaND = 0.125;
% p.alphaD  = 0.125;

p.alphaND = 0.5;
p.alphaD  = 0.5;
p.f       = 0.3;

% pdef_LR = 0.03;
%pdef_LR = 0.03/12;
pdef_LR = 1 - (1 - 0.03)^(1/12);

Nlist = 101;

pdef_SR_list = linspace(0,0.4, Nlist)';
% pdef_SR_list = linspace(0,0.5, Nlist)';

%% recovery function

% use recovery function: 

% recovery_fun_SR = @(pd) max(interp1([0 0.4],p.f + [0.2 -0.2],pd,'linear','extrap'),0);
% recovery_fun_SR = @(pd) max(p.f - pd,0);
recovery_fun_SR = @(pd) p.f*exp(-(pd-pdef_LR));

% plot
figure
fplot(recovery_fun_SR,[0 max(pdef_SR_list)])
xlabel('def prob')
ylabel('recovery, f')

%% base

check_tol = 1e-2;

ell_ND1 = NaN*zeros(Nlist,1);
proploss_ND1 = NaN*zeros(Nlist,1);

[qND_H_LR,qND_L_LR,qD_H_LR,qD_L_LR,qS_ND_LR,qS_D_LR] = CalcBondPrices_LongRun_LCP(p,pdef_LR);

for i = 1:Nlist
    
    pdef_SR = pdef_SR_list(i);
    ptemp = p;
    ptemp.f = recovery_fun_SR(pdef_SR);
    
    [qND_H_SR,qND_L_SR,qS_ND_SR,qD_H_SR,qD_L_SR,qS_D_SR,check] = CalcBondPrices_ShortRun_LCP(ptemp,pdef_SR,qND_H_LR,qND_L_LR);
    
    if check>check_tol; error('Check CalcBondPrices_ShortRun_LCP'); end
    
    ell_ND1(i) = (1 - pdef_SR)*(1-p.m)*p.zeta*(qND_H_SR - qND_L_SR)./qND_H_SR + pdef_SR*p.zeta*(qD_H_LR - qD_L_LR)./qND_H_SR;
    proploss_ND1(i) = (qND_H_SR - qND_L_SR)./qND_H_SR;
    
    
end

%% one year maturity

p2 = p;
p2.m = 1;

ell_ND2 = NaN*zeros(Nlist,1);

[qND_H_LR,qND_L_LR,qD_H_LR,qD_L_LR,qS_ND_LR,qS_D_LR] = CalcBondPrices_LongRun_LCP(p2,pdef_LR);

for i = 1:Nlist
    
    pdef_SR = pdef_SR_list(i);
    ptemp = p2;
    ptemp.f = recovery_fun_SR(pdef_SR);
    
    [qND_H_SR,qND_L_SR,qS_ND_SR,qD_H_SR,qD_L_SR,qS_D_SR,check] = CalcBondPrices_ShortRun_LCP(ptemp,pdef_SR,qND_H_LR,qND_L_LR);
    
    if check>check_tol; error('Check CalcBondPrices_ShortRun_LCP'); end
    
    ell_ND2(i) = (1 - pdef_SR)*(1-p2.m)*p2.zeta*(qND_H_SR - qND_L_SR)./qND_H_SR + pdef_SR*p2.zeta*(qD_H_LR - qD_L_LR)./qND_H_SR;
    
end

%% no short run recovery

ell_ND3 = NaN*zeros(Nlist,1);

[qND_H_LR,qND_L_LR,qD_H_LR,qD_L_LR,qS_ND_LR,qS_D_LR] = CalcBondPrices_LongRun_LCP(p,pdef_LR);

% f_SR = 0;
% f_SR = 0.01;
% f_SR = 0.1;
f_SR = p.f; % long run recovery

p3 = p;
p3.f = f_SR;

for i = 1:Nlist
    
    pdef_SR = pdef_SR_list(i);
    
    [qND_H_SR,qND_L_SR,qS_ND_SR,qD_H_SR,qD_L_SR,qS_D_SR,check] = CalcBondPrices_ShortRun_LCP(p3,pdef_SR,qND_H_LR,qND_L_LR);
    
    if check>check_tol; error('Check CalcBondPrices_ShortRun_LCP'); end
    
    ell_ND3(i) = (1 - pdef_SR)*(1-p3.m)*p3.zeta*(qND_H_SR - qND_L_SR)./qND_H_SR + pdef_SR*p3.zeta*(qD_H_LR - qD_L_LR)./qND_H_SR;
    
end

%% no short + long-run run recovery

p4 = p;
p4.f = f_SR;

ell_ND4 = NaN*zeros(Nlist,1);

[qND_H_LR,qND_L_LR,qD_H_LR,qD_L_LR,qS_ND_LR,qS_D_LR] = CalcBondPrices_LongRun_LCP(p4,pdef_LR);

for i = 1:Nlist
    
    pdef_SR = pdef_SR_list(i);
    
    [qND_H_SR,qND_L_SR,qS_ND_SR,qD_H_SR,qD_L_SR,qS_D_SR,check] = CalcBondPrices_ShortRun_LCP(p4,pdef_SR,qND_H_LR,qND_L_LR);
    
    if check>check_tol; error('Check CalcBondPrices_ShortRun_LCP'); end
    
    ell_ND4(i) = (1 - pdef_SR)*(1-p4.m)*p4.zeta*(qND_H_SR - qND_L_SR)./qND_H_SR + pdef_SR*p4.zeta*(qD_H_LR - qD_L_LR)./qND_H_SR;
    
end

%% maturity extension mechanism: short run probability of reentry, version 1

ell_ND5 = NaN*zeros(Nlist,1);
proploss_ND5 = NaN*zeros(Nlist,1);

[qND_H_LR,qND_L_LR,qD_H_LR,qD_L_LR,qS_ND_LR,qS_D_LR] = CalcBondPrices_LongRun_LCP(p,pdef_LR);

p5 = p;
p5.theta = 1;

for i = 1:Nlist
    
    pdef_SR = pdef_SR_list(i);
    ptemp = p5;
    ptemp.f = recovery_fun_SR(pdef_SR);
    
    [qND_H_SR,qND_L_SR,qS_ND_SR,qD_H_SR,qD_L_SR,qS_D_SR,check] = CalcBondPrices_ShortRun_LCP(ptemp,pdef_SR,qND_H_LR,qND_L_LR);
    
    if check>check_tol; error('Check CalcBondPrices_ShortRun_LCP'); end
    
    ell_ND5(i) = (1 - pdef_SR)*(1-p5.m)*p5.zeta*(qND_H_SR - qND_L_SR)./qND_H_SR + pdef_SR*p5.zeta*(qD_H_LR - qD_L_LR)./qND_H_SR;
    proploss_ND5(i) = (qND_H_SR - qND_L_SR)./qND_H_SR;
    
end

%% maturity extension mechanism, version 2

theta_list = linspace(1/80,1,Nlist);
ell_ND6 = NaN*zeros(Nlist,1);
proploss_ND6 = NaN*zeros(Nlist,1);

[qND_H_LR,qND_L_LR,qD_H_LR,qD_L_LR,qS_ND_LR,qS_D_LR] = CalcBondPrices_LongRun_LCP(p,pdef_LR);

% pdef_SR = pdef_LR;
% pdef_SR = 0.05;
pdef_SR = 0.4;

for i = 1:Nlist
    
    p6 = p;
    p6.theta = theta_list(i);
    ptemp = p6;
    ptemp.f = recovery_fun_SR(pdef_SR);
    
    [qND_H_SR,qND_L_SR,qS_ND_SR,qD_H_SR,qD_L_SR,qS_D_SR,check] = CalcBondPrices_ShortRun_LCP(ptemp,pdef_SR,qND_H_LR,qND_L_LR);
    
    if check>check_tol; error('Check CalcBondPrices_ShortRun_LCP'); end
    
    ell_ND6(i) = (1 - pdef_SR)*(1-p6.m)*p6.zeta*(qND_H_SR - qND_L_SR)./qND_H_SR + pdef_SR*p6.zeta*(qD_H_LR - qD_L_LR)./qND_H_SR;
    proploss_ND6(i) = (qND_H_SR - qND_L_SR)./qND_H_SR;
    
end

%% plots

FontSize = 16;
LineWidth = 2;

figure
hold on
plot(pdef_SR_list, (1 + ell_ND1).^12 - 1, '-b', 'LineWidth', LineWidth)
plot(pdef_SR_list, (1 + ell_ND2).^12 - 1, '--r', 'LineWidth', LineWidth)
plot(pdef_SR_list, (1 + ell_ND3).^12 - 1, ':k', 'LineWidth', LineWidth)
plot(pdef_SR_list, (1 + ell_ND4).^12 - 1, '-.m', 'LineWidth', LineWidth)
xlabel('One period ahead default probablity, $p_d$', 'Interpreter', 'Latex', 'FontSize', FontSize)
ylabel('Liq. premium $\ell_{ND}$, annualized', 'Interpreter', 'Latex', 'FontSize', FontSize)
legend({'Long-term debt + recovery','Short-term debt','No short run recovery only','No short and long run recovery'},'Location','NorthWest', 'Interpreter', 'Latex', 'FontSize', FontSize)
legend boxoff

%% 1x2 plot

FontSize = 22;
LineWidth = 2;

figure

subplot(1,2,1)
hold on
plot(pdef_SR_list, (1 + ell_ND1).^12 - 1, '-b', 'LineWidth', LineWidth)
plot(pdef_SR_list, (1 + ell_ND2).^12 - 1, '--r', 'LineWidth', LineWidth)
xlabel('One period ahead default probablity, $p_d$', 'Interpreter', 'Latex', 'FontSize', FontSize)
ylabel('$\ell_{ND}$, annualized', 'Interpreter', 'Latex', 'FontSize', FontSize)
legend({['Long-term debt, $1/m=60$ mos.'],['Short-term debt, $1/m=1$ mo.']},'Location','NorthWest', 'Interpreter', 'Latex', 'FontSize', FontSize)
legend boxoff
title('A. Effect of maturity', 'Interpreter', 'Latex', 'FontSize', FontSize)
box on

subplot(1,2,2)
hold on
plot(pdef_SR_list, (1 + ell_ND1).^12 - 1, '-b', 'LineWidth', LineWidth)
plot(pdef_SR_list, (1 + ell_ND3).^12 - 1, '--r', 'LineWidth', LineWidth)
% plot(pdef_SR_list, (1 + ell_ND4).^12 - 1, ':k', 'LineWidth', LineWidth)
xlabel('One period ahead default probablity, $p_d$', 'Interpreter', 'Latex', 'FontSize', FontSize)
ylabel('$\ell_{ND}$, annualized', 'Interpreter', 'Latex', 'FontSize', FontSize)
legend({['Decreasing recovery, $f=f(p_d)$'],['Constant recovery, $f=\overline{f}$']},'Location','NorthWest', 'Interpreter', 'Latex', 'FontSize', FontSize)
legend boxoff
title('B. Effect of recovery', 'Interpreter', 'Latex', 'FontSize', FontSize)
box on

% illus_ingredients_jump_to_def.eps

%% 1x3 plot

figure

subplot(1,2,1)
hold on
plot(pdef_SR_list, (1 + ell_ND1).^12 - 1, '-b', 'LineWidth', LineWidth)
plot(pdef_SR_list, (1 + ell_ND2).^12 - 1, '--r', 'LineWidth', LineWidth)
xlabel('One period ahead default probablity, $p_d$', 'Interpreter', 'Latex', 'FontSize', FontSize)
ylabel('$\ell_{ND}$, annualized', 'Interpreter', 'Latex', 'FontSize', FontSize)
legend({['Long-term debt, $m=',num2str(p.m),'$'],['Short-term debt, $m=1$']},'Location','NorthWest', 'Interpreter', 'Latex', 'FontSize', FontSize)
legend boxoff
title('A. Effect of maturity', 'Interpreter', 'Latex', 'FontSize', FontSize)
box on

subplot(1,2,2)
hold on
plot(pdef_SR_list, (1 + ell_ND1).^12 - 1, '-b', 'LineWidth', LineWidth)
plot(pdef_SR_list, (1 + ell_ND3).^12 - 1, '--r', 'LineWidth', LineWidth)
xlabel('One period ahead default probablity, $p_d$', 'Interpreter', 'Latex', 'FontSize', FontSize)
ylabel('$\ell_{ND}$, annualized', 'Interpreter', 'Latex', 'FontSize', FontSize)
legend({['Positive recovery, $f=',num2str(p.f),'$'],['No recovery, $f=0$']},'Location','NorthWest', 'Interpreter', 'Latex', 'FontSize', FontSize)
legend boxoff
title('B. Effect of recovery', 'Interpreter', 'Latex', 'FontSize', FontSize)
box on

%% maturity extension mechanism

figure

subplot(1,2,1)
plot(1./theta_list/12, (1 + ell_ND6).^12 - 1, '-b', 'LineWidth', LineWidth)
xlabel('Expected time in autarky $1/\theta$, years', 'Interpreter', 'Latex', 'FontSize', FontSize)
ylabel('$\ell_{ND}$, annualized', 'Interpreter', 'Latex', 'FontSize', FontSize)
title('Maturity extension mechanism', 'Interpreter', 'Latex', 'FontSize', FontSize)

subplot(1,2,2)
hold on
plot(pdef_SR_list, (1 + ell_ND1).^12 - 1, '-b', 'LineWidth', LineWidth)
plot(pdef_SR_list, (1 + ell_ND5).^12 - 1, '--r', 'LineWidth', LineWidth)
xlabel('One period ahead default probablity, $p_d$', 'Interpreter', 'Latex', 'FontSize', FontSize)
ylabel('$\ell_{ND}$, annualized', 'Interpreter', 'Latex', 'FontSize', FontSize)
legend({['Long time in autarky, $\theta=',num2str(p.theta),'$'],['Short time in autarky, $\theta=1$']},'Location','NorthWest', 'Interpreter', 'Latex', 'FontSize', FontSize)
legend boxoff
title('C. Maturity extension mechanism', 'Interpreter', 'Latex', 'FontSize', FontSize)
box on

%% 

figure

subplot(1,2,1)
plot(1./theta_list, proploss_ND6, '-b', 'LineWidth', LineWidth)
xlabel('Expected time in autarky $1/\theta$, mos.', 'Interpreter', 'Latex', 'FontSize', FontSize)
ylabel('$(q_{ND}^H-q_{ND}^L)/q_{ND}^H$', 'Interpreter', 'Latex', 'FontSize', FontSize)
title('A. Effect of time in autarky', 'Interpreter', 'Latex', 'FontSize', FontSize)
legend({['$p_d=',num2str(pdef_SR),'$']},'Location','NorthWest', 'Interpreter', 'Latex', 'FontSize', FontSize)
legend boxoff

subplot(1,2,2)
hold on
plot(pdef_SR_list, proploss_ND1, '-b', 'LineWidth', LineWidth)
plot(pdef_SR_list, proploss_ND5, '--r', 'LineWidth', LineWidth)
xlabel('One period ahead default probablity, $p_d$', 'Interpreter', 'Latex', 'FontSize', FontSize)
ylabel('$(q_{ND}^H-q_{ND}^L)/q_{ND}^H$', 'Interpreter', 'Latex', 'FontSize', FontSize)
legend({['$1/\theta=',num2str(round(1/p.theta)),'$ mos.'],['$1/\theta=1$ mo.']},'Location','NorthWest', 'Interpreter', 'Latex', 'FontSize', FontSize)
legend boxoff
title('B. Maturity extension mechanism', 'Interpreter', 'Latex', 'FontSize', FontSize)
box on

% illus_mat_extension_jump_to_def.eps