#include "liquidity_defs.h"
#include "liquidity_vfi.h"
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <cstring>
#include <sstream>
#include <omp.h>

int main(int argc, char* argv[])
{

	// inputs:
	// 1. parameter_folder
	// 2. parameter prefix
	// 3. N, number of parameters
	// 4. output_folder_prefix
	// e.g. ./batch batchparms param 10 outputs

	// input folder consists of 
	// 1. FILE_LIST.txt-- list of names
	// 2. parms1.txt, parms2.txt, ...

	char *paramfoldername, *paramprefix, *outputfolderprefix;
	int NumParamSets;
	if (argc != 5)
	{
		std::cout << "Error: program expects 4 inputs for the parameter file." << std::endl;
		std::cout << "Format: ./runprog [Parameter Folder] [Parameter prefix] [N] [Output folder prefix]" << std::endl;
		exit (EXIT_FAILURE);
	}
	else
	{
		paramfoldername = argv[1];
		paramprefix = argv[2]; // e.g. param
		NumParamSets = atoi(argv[3]); // N. Parameter sets are paramx.txt where x=1,...,N
		outputfolderprefix = argv[4];
	}

	// set up input and output file names
	std::string *InputParamNames = new std::string[NumParamSets];
	std::string *OutputFolderNames = new std::string[NumParamSets];

	std::string s_paramfoldername, s_paramprefix, s_outputfolderprefix;
	s_paramfoldername.assign(paramfoldername);
	s_paramprefix.assign(paramprefix);
	s_outputfolderprefix.assign(outputfolderprefix);

	std::ostringstream converter;

	for (int i = 0; i < NumParamSets; i++)
	{

		converter.str("");
		converter << i+1;

		InputParamNames[i] = s_paramfoldername + "/" + s_paramprefix + converter.str() + ".txt";
		OutputFolderNames[i]  = s_outputfolderprefix + s_paramprefix + converter.str();

	}
	

	int nDevices, cpu_tid, idxfile;
  	cudaGetDeviceCount(&nDevices);
	
	parms_bsl_mod p;
	char *cstr;

	#pragma omp parallel num_threads(nDevices) default(none)\
	shared(NumParamSets, InputParamNames, OutputFolderNames)\
	private(cpu_tid, idxfile, p, cstr)
	{

		cpu_tid = omp_get_thread_num();
		cudaSetDevice(cpu_tid);

		#pragma omp for schedule(dynamic)
		for (idxfile = 0; idxfile<NumParamSets; idxfile++)
		{

			cstr = new char[InputParamNames[idxfile].length() + 1];
			strcpy(cstr, InputParamNames[idxfile].c_str());

			if (p.readparms(cstr) == EXIT_SUCCESS)
			{
				// solve model and save
				SolveModel(p, OutputFolderNames[idxfile].c_str(), cpu_tid);
				// delete p
				p.clean();
			}
			delete[] cstr;

		}

	}

	return EXIT_SUCCESS;

}