addpath(genpath('../matlab'))
addpath('../src_mex')
clear all

%% parameters

p.rra = 2;
p.beta = 0.983965796101421;
p.clb = 1e-6;

p.ggq_mstd = 0.004;
p.ggq_mlb = -3*p.ggq_mstd;
p.ggq_mub = 3*p.ggq_mstd;
p.ggq_mmean = 0.5*(p.ggq_mub + p.ggq_mlb);
p.ggq_mDval = p.ggq_mmean;

p.errtol_bisect = 1e-12;
p.maxiters_bisect = 1000;
p.r = 0.0033;
p.holding_cost = 0.005746714158803;
p.prob_liqshock = 0.1393;
p.prob_meet_mkt_maker_ND = 0.8647;
p.prob_meet_mkt_maker_D = 0.8647;
p.bargain_power_mkt_maker_ND = 0.5;
p.bargain_power_mkt_maker_D = 0.5;
p.reenterprob = 0.0128;
p.debt_z = 0.01;
p.debt_m = 0.0167;
p.debt_writedown_reentry = 0;
p.max_def_prob = 0.75;
p.recovery_bmax = 0.797077101425576;
% p.gridsize_tfp = 200;
% p.gridsize_b = 450;
p.gridsize_tfp = 64;
p.gridsize_b = 256;
p.bmax = 6;
p.grid_b = linspace(0,p.bmax,p.gridsize_b)';
p.rho_tfp = 0.9830;
p.sigma_tfp = 0.0151;

p.width_std_tfp = 3;
p.conv_chk_width_std_tfp = 2.5;
p.conv_chk_b_ub = 5.5;

p.d_y = -0.245415825478762;
p.d_yy = 0.316268295265247;

% p.maxiters = 10000;
p.maxiters = 5000; % shorten for testing
p.errtolV = 1e-6;
p.errtolQ = 1e-5;
p.NumUpdateRegimes = 5;
p.UpdateRegimeIter = [200;500;1000;2000];
p.UpdateWeightOldV = zeros(5,1);
p.UpdateWeightOldQ = [0;0.5;0.9;0.95;0.98];
p.UpdateWeightOldDefProb = [0;0;0;0;0.9];

% p.gridsize_mpr = 1;
p.gridsize_mpr = 5;
% p.width_std_mpr = 3;
p.width_std_mpr = 2;

p.conv_chk_width_std_mpr = 2.5;

p.mean_mpr = 0.1804;
p.rho_mpr = 0.9885;
p.sigma_mpr = 0.0693;
p.corr_sdf_tfp_shock = 0.4;

%% solve model on GPU

useDevice = 0;
DisplayInterval = 100;

tic
model_gpu_extended = mexSolveExtendedModelGivenParms(p, useDevice, DisplayInterval);
toc

%% get policies on CPU

tic
model_gpu_extended_converted = ConvertCPPtoMAT(p,model_gpu_extended);
[model,p] = ProcessMexGPUOutputs_liq(p, model_gpu_extended_converted);
toc

%% simulate

TBurn = 12*10^3;
T = TBurn + 12*10^5;
QuarterLength = 3;
NumQuartersExcludeAfterReentry = 20;

rng(0);
Us_z = rand(T,1);
Us_m = rand(T,1);
Us_x = rand(T,1);
Us_reenter = rand(T,1);

tic
[~,stats_rng0] = SimSinglePath(p,model,T,TBurn,Us_z,Us_x,Us_m,Us_reenter,QuarterLength,NumQuartersExcludeAfterReentry);
toc

%% simulate multiple paths

% NumPaths = 1000;
NumPaths = 100;

stats = SimModelStats(p,model,NumPaths,T,TBurn,QuarterLength,NumQuartersExcludeAfterReentry);

%% save

% save('test.mat','p','model_gpu','model_gpu_converted','model','stats_rng0','stats')