function mod = ComputeModelAuxiliaryEfficient(p,mod)
% things to compute:
% 1. term structure of default probabilities
    
    [qND_noliq,qD_noliq] = CalcBondPrices_noliq(p,mod);
    
    mod.qND_noliq = qND_noliq;
    mod.qD_noliq = qD_noliq;

end

function [qND_noliq,qD_noliq] = CalcBondPrices_noliq(p,model)

    qND_noliq = zeros(p.gridsize_tfp*p.gridsize_mpr,p.gridsize_b);
    qD_noliq = zeros(p.gridsize_tfp*p.gridsize_mpr,p.gridsize_b);
    qND_noliq_old = zeros(p.gridsize_tfp*p.gridsize_mpr,p.gridsize_b);
    qD_noliq_old = zeros(p.gridsize_tfp*p.gridsize_mpr,p.gridsize_b);
    qND_noliq_haircut = zeros(p.gridsize_tfp*p.gridsize_mpr,p.gridsize_b);
    
    bContinue = true;
    iter = 0;
    
    default_threshold_eps = reshape(model.defthresh,[p.gridsize_tfp*p.gridsize_mpr p.gridsize_b]);
    debt_policy = reshape(model.policy,[p.gridsize_tfp*p.gridsize_mpr p.gridsize_b]);
    
    rec_frac = min(1-p.debt_writedown_reentry,p.recovery_bmax./p.grid_b);
    rec_frac = permute(repmat(reshape(rec_frac,[],1),[1 p.gridsize_tfp*p.gridsize_mpr]),[2 1]);
    
    [const_qupdate,wND_qupdate,wD_qupdate] = SetUpWeights(p,default_threshold_eps,debt_policy);
    
    while bContinue
        
        % integrate over noise
        for idx_y_mpr = 1:p.gridsize_tfp*p.gridsize_mpr
            qND_noliq(idx_y_mpr,:) = CalcBondPrices_noliq_given_y(p,const_qupdate(:,idx_y_mpr),wND_qupdate(:,:,idx_y_mpr),wD_qupdate(:,idx_y_mpr),...
                    qND_noliq_old(idx_y_mpr,:),qD_noliq_old(idx_y_mpr,:));
        end
        
        % z'|z
        qND_noliq = p.prob_RN_y_mpr*qND_noliq/(1+p.r);
        qD_noliq1 = p.prob_RN_y_mpr*qD_noliq_old;
        
        for idx_y_mpr = 1:p.gridsize_tfp*p.gridsize_mpr
            qND_noliq_haircut(idx_y_mpr,:) = interp1(p.grid_b,qND_noliq(idx_y_mpr,:),min((1-p.debt_writedown_reentry)*p.grid_b,p.recovery_bmax),'nearest');
        end

        qD_noliq = max(0,(1-p.reenterprob)*qD_noliq1/(1+p.r) ...
            + p.reenterprob*rec_frac.*qND_noliq_haircut);
        
        iter = iter + 1;
        
        err_qND = max(abs(reshape(qND_noliq-qND_noliq_old,[],1)));
        err_qD = max(abs(reshape(qD_noliq-qD_noliq_old,[],1)));
        
        if iter>p.maxiters || max([err_qND err_qD])<p.errtolQ
            bContinue = false;
        else
            qND_noliq_old = qND_noliq;
            qD_noliq_old = qD_noliq;
        end
        
        fprintf('iter %g, err(qND)=%g, err(qD)=%g.\n', iter, err_qND, err_qD);
        
    end
    
    qND_noliq = reshape(qND_noliq,[p.gridsize_tfp p.gridsize_mpr p.gridsize_b]);
    qD_noliq = reshape(qD_noliq,[p.gridsize_tfp p.gridsize_mpr p.gridsize_b]);

end

function q = CalcBondPrices_noliq_given_y(p,const,wND,wD,qND,qD)
% const is bx1
% wND is b'xb
% wD is bx1

%     q = zeros(1,p.gridsize_b);
%     
%     for idx_b = 1:p.gridsize_b
%         q(idx_b) = CalcBondPrices_noliq_given_y_b(p,default_threshold_eps(idx_b),debt_policy{idx_b},qND,qD(idx_b));
%     end
    qND = reshape(qND,[p.gridsize_b 1]);
    qD = reshape(qD,[p.gridsize_b 1]);

    q = const + transpose(wND)*qND + wD.*qD;

end

function [const,wND,wD] = SetUpWeights(p,default_threshold_eps,debt_policy)

    const = zeros(p.gridsize_b,p.gridsize_tfp*p.gridsize_mpr);
    wND = zeros(p.gridsize_b,p.gridsize_b,p.gridsize_tfp*p.gridsize_mpr);
    wD = zeros(p.gridsize_b,p.gridsize_tfp*p.gridsize_mpr);
    
    parfor idx_y_mpr = 1:p.gridsize_tfp*p.gridsize_mpr
        [const(:,idx_y_mpr),wND(:,:,idx_y_mpr),wD(:,idx_y_mpr)] = SetUpWeights_given_y_mpr(p,default_threshold_eps(idx_y_mpr,:),debt_policy(idx_y_mpr,:));
    end

end

function [const,wND,wD] = SetUpWeights_given_y_mpr(p,default_threshold_eps,debt_policy)
    
    const = zeros(p.gridsize_b,1);
    wND = zeros(p.gridsize_b,p.gridsize_b);
    wD = zeros(p.gridsize_b,1);
    
    for idx_b = 1:p.gridsize_b
        [const(idx_b),wND(:,idx_b),wD(idx_b)] = SetUpWeights_given_y_mpr_b(p,default_threshold_eps(idx_b),debt_policy{idx_b});
    end

end

function [const,wND,wD] = SetUpWeights_given_y_mpr_b(p,default_threshold_eps,debt_policy)
% update is now: q' = const + wND*qND + wD*qD

    const = 0;
    wND = zeros(p.gridsize_b,1); % vector
    wD = 0; % scalar
    
    if ~debt_policy.EmptyChoiceSet
        for j = 1:debt_policy.NumRegions
            region_lb = debt_policy.eps_mesh(j);
            region_ub = debt_policy.eps_mesh(j+1);
            % update ceC and qND_H and qND_L
            if default_threshold_eps>=region_ub
                % default in the entire region
                mass = normcdf(region_ub,p.ggq_mmean,p.ggq_mstd) - ...
                    normcdf(region_lb,p.ggq_mmean,p.ggq_mstd);
                wD = wD + mass;
            else
                idx_bnext = debt_policy.idx_bnext(j);
                if default_threshold_eps<=region_lb
                    % no default in the entire region
                    % f(Clb,RRA,BETA,C,W)
                    mass = normcdf(region_ub,p.ggq_mmean,p.ggq_mstd) - ...
                        normcdf(region_lb,p.ggq_mmean,p.ggq_mstd);
                    const = const + (p.debt_m+(1-p.debt_m)*p.debt_z)*mass;
                    wND(idx_bnext) = wND(idx_bnext) + (1-p.debt_m)*mass;
                else
                    % default then no default
                    massdef = normcdf(default_threshold_eps,p.ggq_mmean,p.ggq_mstd) - ...
                        normcdf(region_lb,p.ggq_mmean,p.ggq_mstd);
                    massnodef = normcdf(region_ub,p.ggq_mmean,p.ggq_mstd) - ...
                        normcdf(default_threshold_eps,p.ggq_mmean,p.ggq_mstd);
                    const = const + (p.debt_m+(1-p.debt_m)*p.debt_z)*massnodef;
                    wND(idx_bnext) = wND(idx_bnext) + (1-p.debt_m)*massnodef;
                    wD = wD + massdef;
                end
            end
        end
        const = const/p.ggq_mmass;
        wD = wD/p.ggq_mmass;
        wND = wND/p.ggq_mmass;
    else
        % debt_policy.EmptyChoiceSet = true.
        % so default throughout
        wD = 1;
    end

end