set(0,'DefaultFigureWindowStyle','docked')

clear

%% specify parameters

p.rra = 2;
p.beta = 0.9841;
p.clb = 1e-6;

p.ggq_mstd = 0.004;
p.ggq_mlb = -3*p.ggq_mstd;
p.ggq_mub = 3*p.ggq_mstd;
p.ggq_mmean = 0.5*(p.ggq_mub + p.ggq_mlb);
p.ggq_mDval = p.ggq_mmean;

p.errtol_bisect = 1e-12;
p.maxiters_bisect = 1000;
p.r = 0.0033;
p.holding_cost = 0.0014;
p.prob_liqshock = 0.1393;
p.prob_meet_mkt_maker_ND = 0.8647;
p.prob_meet_mkt_maker_D = 0.8647;
p.bargain_power_mkt_maker_ND = 0.8750;
p.bargain_power_mkt_maker_D = 1;
p.reenterprob = 0.0128;
p.debt_z = 0.01;
p.debt_m = 0.0167;
p.debt_writedown_reentry = 0;
p.max_def_prob = 0.75;
p.recovery_bmax = 0.83;
% p.gridsize_tfp = 64;
p.gridsize_tfp = 128;
p.gridsize_b = 450;
p.bmax = 6;
p.rho_tfp = 0.9830;
p.sigma_tfp = 0.0151;

% uncond_std = p.sigma_tfp/sqrt(1 - p.rho_tfp^2);
% width_numstd = 3;
% width_numstd_conv_chk = 3;

p.width_std_tfp = 3;
p.width_std_mpr = 3;

p.conv_chk_width_std_tfp = 2.5;
p.conv_chk_width_std_mpr = 2.5;
p.conv_chk_b_ub = 5.5;

p.d_y = -0.2640;
p.d_yy = 0.3370;

p.gridsize_mpr = 3;
% p.gridsize_mpr = 7;
% p.gridsize_mpr = 32;
% p.gridsize_mpr = 1;
% p.gridsize_mpr = 5;
p.mean_mpr = 0.1804;
p.rho_mpr = 0.9885;
p.sigma_mpr = 0.0693;
% p.corr_sdf_tfp_shock = 0;
% p.corr_sdf_tfp_shock = 0.05;
p.corr_sdf_tfp_shock = 0.5;
% p.corr_sdf_tfp_shock = 0.99;
% p.corr_sdf_tfp_shock = 1;

p.maxiters = 12000;
p.errtolV = 1e-6;
p.errtolQ = 1e-6;
p.NumUpdateRegimes = 4;
p.UpdateRegimeIter = [200;500;1000];
p.UpdateWeightOldV = zeros(4,1);
p.UpdateWeightOldQ = [0;0.5;0.9;0.95];
p.UpdateWeightOldDefProb = zeros(4,1);

%% read

p_out = mexTestReadParms(p);

%% convert to matlab

p_out.z = log(p_out.grid_y_nd);
p_out.P_y = permute(reshape(p_out.P_y,[p.gridsize_tfp p.gridsize_tfp]),[2 1]);
p_out.P_mpr = permute(reshape(p_out.P_mpr,[p.gridsize_mpr p.gridsize_mpr]),[2 1]);
p_out.Q_y_mpr_ynext = permute(reshape(p_out.Q_y_mpr_ynext,[p.gridsize_tfp p.gridsize_mpr p.gridsize_tfp]),[3 2 1]);

check_err = max(abs(reshape(sum(p_out.Q_y_mpr_ynext,3) - 1,[],1)))

%% plot

[zz1,zz2] = ndgrid(p_out.z,p_out.z);
[xx1,xx2] = ndgrid(p_out.grid_mpr,p_out.grid_mpr);

figure
surf(zz1,zz2,p_out.P_y)
xlabel('z')
ylabel('z''')
zlabel('p(z''|z)')
set(gca,'xlim',p_out.z([1 end]))
set(gca,'ylim',p_out.z([1 end]))

if p.gridsize_mpr>1
    figure
    surf(xx1,xx2,p_out.P_mpr)
    xlabel('x')
    ylabel('x''')
    zlabel('p(z''|z)')
    set(gca,'xlim',p_out.grid_mpr([1 end]))
    set(gca,'ylim',p_out.grid_mpr([1 end]))
end

if p.gridsize_mpr==1
    figure
    subplot(1,3,1)
    surf(zz1,zz2,squeeze(p_out.Q_y_mpr_ynext))
    xlabel('z')
    ylabel('z''')
    zlabel('q(z''|z)')
    set(gca,'xlim',p_out.z([1 end]))
    set(gca,'ylim',p_out.z([1 end]))
    axis square
    
    subplot(1,3,2)
    surf(zz1,zz2,squeeze(p_out.Q_y_mpr_ynext)./p_out.P_y)
    xlabel('z')
    ylabel('z''')
    zlabel('q(z''|z)/q(z''|z)')
    set(gca,'xlim',p_out.z([1 end]))
    set(gca,'ylim',p_out.z([1 end]))
    axis square
    
    [V,D] = eig(p_out.P_y');
    pz = V(:,1)/sum(V(:,1));
    [V,D] = eig(squeeze(p_out.Q_y_mpr_ynext)');
    qz = V(:,1)/sum(V(:,1));
    
    subplot(1,3,3)
    hold on
    plot(p_out.z,pz,'-b')
    plot(p_out.z,qz,'-.r')
    xlabel('z')
    legend({'p(z)','q(z)'})
    legend boxoff
    title('Stationary prob')
    grid on
    axis square
    
else
    
    % stationary distribution, ignore machine zeros
    sparse_i = [];
    sparse_j = [];
    sparse_v = [];
    
    temp = zeros(p.gridsize_tfp,1);
    
    Q_ynext_y_mpr = permute(p_out.Q_y_mpr_ynext,[3 1 2]);
    ones_Ny = ones(p.gridsize_tfp,1);
    IDX_ZNEXT = (1:p.gridsize_tfp)';
    
    for idx1 = 1:p.gridsize_tfp*p.gridsize_mpr
        [idx_z, idx_x] = ind2sub([p.gridsize_tfp p.gridsize_mpr],idx1);
        for idx_xnext = 1:p.gridsize_mpr
            temp(:) = p_out.P_mpr(idx_x,idx_xnext).*Q_ynext_y_mpr(:,idx_z,idx_x);
            idx_select = find(temp~=0);
            n_select = numel(idx_select);
            if n_select>0
                sparse_i = [sparse_i; idx1*ones_Ny(1:n_select)];
                sparse_j = [sparse_j; sub2ind([p.gridsize_tfp p.gridsize_mpr],IDX_ZNEXT(idx_select),idx_xnext*ones_Ny(1:n_select))];
                sparse_v = [sparse_v; temp(idx_select)];
            end
        end
    end
    
    sp_trans_mat = sparse(sparse_i,sparse_j,sparse_v,p.gridsize_tfp*p.gridsize_mpr,p.gridsize_tfp*p.gridsize_mpr);
    
    [V,D] = eigs(sp_trans_mat',1);
    fprintf('D(1)=%g.\n',D(1));
    q_zx = reshape(V(:,1)/sum(V(:,1)),[p.gridsize_tfp p.gridsize_mpr]);
    
    [zz,xx] = ndgrid(p_out.z,p_out.grid_mpr);
    
    figure
    subplot(1,2,1)
    surf(zz,xx,q_zx)
    xlabel('z')
    ylabel('x')
    title('q(z,x)')
    set(gca,'xlim',p_out.z([1 end]))
    set(gca,'ylim',p_out.grid_mpr([1 end]))
    axis square
    
    qcond = q_zx./permute(repmat(reshape(sum(q_zx,1),[],1),[1 p.gridsize_tfp]),[2 1]);
    
    subplot(1,2,2)
    hold on
    plot(p_out.z,qcond(:,1),'-b','LineWidth',2)
    plot(p_out.z,qcond(:,end),'-r','LineWidth',2)
    plot(p_out.z,qcond(:,2:end-1))
    xlabel('z')
    axis square
    title('q(z|x)')
    legend({'q(z|x_{min})','q(z|x_{max})'})
    legend boxoff
    
end