
*----------------------------------------------------------------------------------------
*0. Inputs
*----------------------------------------------------------------------------------------

clear all
set more off
cd "C:\Users\juanp\Dropbox\Proyectos\3.Submitted\PX.Liquidity.sovereign.debt.markets\sovereign.liquidity\Notes\Replication.PX.22"

*----------------------------------------------------------------------------------------
*1. Loading data
*----------------------------------------------------------------------------------------

use panel_bonds_19932004, clear

replace AmountIssued=int(AmountIssued)

label var ba_spread "Proportional bid-ask spread"

*-----------------------------------------------------------------------------------------
*2. Descriptive Statistics.
*-----------------------------------------------------------------------------------------

estpost summ ba_spread, d
esttab using "ba_spread_allsample.tex", label booktabs ///
title(Descriptive Stats: All sample\label {tab1}) cells("mean(fmt(a2)) sd(fmt(a2)) p25(fmt(a2)) p50(fmt(a2)) p75(fmt(a2)) min(fmt(a2)) max(fmt(a2))") ///
stats(N, fmt(%5.0f) labels("N")) replace nonum nomtitle collabels("Mean" "SD" "P25" "Median" "P75" "Min." "Max.")


*-----------------------------------------------------------------------------------------
*5. How many bonds?
*-----------------------------------------------------------------------------------------

distinct ISIN //32 bonds, as expected.

*-----------------------------------------------------------------------------------------
*6. Do all bonds have data after they are issued?
*-----------------------------------------------------------------------------------------

*Verify by bid price, WLOG.

bys idpanel: gen total_period=_N

gen aux=1-missing(px_bid)

*Running sum of nonmissing observations

rangestat (sum) total_nonmiss=aux, interval(aux . 0) by(idpanel)

*Identifier if an observation is missing after data "started":

gen indic=aux==0 if total_nonmiss>0

tab indic

*After data "started", there are no missing observations, therefore, all bonds have data after they are issued.

drop total_period aux total_nonmiss indic

*-----------------------------------------------------------------------------------------
*7. Descriptive Statistics for all bonds, weighted by amount issued
*-----------------------------------------------------------------------------------------

collapse (mean) ba_spread [fw=AmountIssued], by(date)

label var ba_spread "Proportional bid-ask spread"

estpost summ ba_spread, d
esttab using "ba_spread.tex", label booktabs ///
title(Descriptive Stats: All sample (weighted by amount issued)\label {tab1}) cells("mean(fmt(a2)) sd(fmt(a2)) p25(fmt(a2)) p50(fmt(a2)) p75(fmt(a2)) min(fmt(a2)) max(fmt(a2))") ///
stats(N, fmt(%5.0f) labels("N")) replace nonum nomtitle collabels("Mean" "SD" "P25" "Median" "P75" "Min." "Max.")


*-----------------------------------------------------------------------------------------
*8. Bid-ask spreads by subsamples: Default
*-----------------------------------------------------------------------------------------

gen sample1=date>=date("03/12/2001", "DMY")

label var sample1 "03/12/2001 - 31/12/2004"

local xtext : variable label sample1 

estpost summ ba_spread if sample1==1, d
esttab using "ba_spread_1.tex", label booktabs ///
title(Descriptive Stats:- `xtext' (weighted by amount issued) \label {tab1}) cells("mean(fmt(a2)) sd(fmt(a2)) p25(fmt(a2)) p50(fmt(a2)) p75(fmt(a2)) min(fmt(a2)) max(fmt(a2))") ///
stats(N, fmt(%5.0f) labels("N")) replace nonum nomtitle collabels("Mean" "SD" "P25" "Median" "P75" "Min." "Max.")


*-----------------------------------------------------------------------------------------
*9. Bid-ask spreads by subsamples: Pre default
*-----------------------------------------------------------------------------------------

gen sample2=date<date("03/12/2001", "DMY")

label var sample2 "01/01/1993 - 03/12/2001"

local xtext : variable label sample2 

estpost summ ba_spread if sample2==1, d
esttab using "ba_spread_2.tex", label booktabs ///
title(Descriptive Stats:- `xtext' (weighted by amount issued) \label {tab1}) cells("mean(fmt(a2)) sd(fmt(a2)) p25(fmt(a2)) p50(fmt(a2)) p75(fmt(a2)) min(fmt(a2)) max(fmt(a2))") ///
stats(N, fmt(%5.0f) labels("N")) replace nonum nomtitle collabels("Mean" "SD" "P25" "Median" "P75" "Min." "Max.")

